@@ -594,10 +594,12 @@ class _LinkedDataGenerator:
 
     def scenario_1(self):
         '''
-        Values in all linked columns are drawn from a uniform distribution
+        Values in all linked columns are drawn from a uniform distribution,
+        excluding Missing data which is in SQL DB, last. Potentially need to
+        pop it and reinsert since order isn't always guaranteed.
         '''
 
-        idx = self.rng.choice(len(self.sql_df), self.num_rows)
+        idx = self.rng.choice(len(self.sql_df) - 1, self.num_rows)
 
         anon_list = [self.sql_df.iloc[x, :].values for x in idx]
 
@@ -619,6 +621,9 @@ class _LinkedDataGenerator:
         base_col_vals = None
         base_col_df = self.spec_dict["columns"][self.base_col]["original_values"][:-1]
         base_col_prob = np.array(base_col_df["probability_vector"]).astype(float)
+        
+        if base_col_prob.sum() != 1:
+            base_col_prob /= base_col_prob.sum()
 
         if self.anon_set != "random":
             #replace original_values with anonymised aliases for weights_table
@@ -682,6 +687,9 @@ class _LinkedDataGenerator:
         base_col_df = self.spec_dict["columns"][self.base_col]["original_values"][:-1]
         base_col_prob = np.array(base_col_df["probability_vector"]).astype(float)
 
+        if base_col_prob.sum() != 1:
+            base_col_prob /= base_col_prob.sum()
+
         if self.anon_set != "random":
 
             orig_df = self.spec_dict["columns"][self.base_col]["original_values"]
@@ -729,7 +737,7 @@ class _LinkedDataGenerator:
             for c in self.linked_cols:
                 
                 #just generate a DF with duplicate paired columns
-                for pair in self.spec_dict["columns"][c]["paired_columns"]:
+                for pair in self.spec_dict["columns"][c]["paired_columns"] or list():
 
                     #overwrite linked_df
                     linked_df = pd.concat(
