@@ -11,7 +11,7 @@ import pandas as pd
 import numpy as np
 
 # Exhibit imports
-from exhibit.core.constants import ORIGINAL_VALUES_DB
+from exhibit.core.constants import MISSING_DATA_STR, ORIGINAL_VALUES_DB
 from exhibit.core.sql import create_temp_table
 from exhibit.db import db_util
 
@@ -134,6 +134,7 @@ class constraintsTests(unittest.TestCase):
         an operator OR a column name, try to parse it as a scalar
         '''
 
+        self.ch.spec_dict["metadata"] = {"numerical_columns": ["A", "B B"]}
         self.ch.spec_dict["columns"] = {
             "A": {
                 "type": "continuous",
@@ -172,6 +173,7 @@ class constraintsTests(unittest.TestCase):
         using weighted uniform parameters (dispersion)
         '''
 
+        self.ch.spec_dict["metadata"] = {"numerical_columns": ["A A", "B"]}
         self.ch.spec_dict["columns"] = {
             "A A": {
                 "type": "continuous",
@@ -193,7 +195,7 @@ class constraintsTests(unittest.TestCase):
 
         test_df = pd.DataFrame(
             data={
-                "A A":[np.NaN, 0, 20, 2, 50],
+                "A A":[pd.NA, 0, 20, 2, 50],
                 "B":[1, 5, 21, 1, 1000]
             }
         )
@@ -213,6 +215,7 @@ class constraintsTests(unittest.TestCase):
         using normal distribution parameters (mean and std)
         '''
 
+        self.ch.spec_dict["metadata"] = {"numerical_columns": ["A"]}
         self.ch.spec_dict["columns"] = {
             "A": {
                 "type": "continuous",
@@ -257,6 +260,7 @@ class constraintsTests(unittest.TestCase):
         modifies the passed-in dataframe in-place.
         '''
 
+        self.ch.spec_dict["metadata"] = {"numerical_columns": ["A", "B", "C"]}
         self.ch.spec_dict["columns"] = {
             "A": {
                 "type": "continuous",
@@ -303,6 +307,7 @@ class constraintsTests(unittest.TestCase):
         involving timeseries column.
         '''
 
+        self.ch.spec_dict["metadata"] = {"numerical_columns": []}
         self.ch.spec_dict["columns"] = {
             "arrival date": {
                 "type": "date",
@@ -342,6 +347,7 @@ class constraintsTests(unittest.TestCase):
         Constraint dependent column values to a date string
         '''
 
+        self.ch.spec_dict["metadata"] = {"numerical_columns": []}
         self.ch.spec_dict["columns"] = {
             "arrival_date": {
                 "type": "date",
@@ -708,7 +714,8 @@ class constraintsTests(unittest.TestCase):
         })
 
         test_gen = tm.ConstraintHandler(test_dict, test_data)
-        result = test_gen.process_constraints()
+        # remember for make_distinct we're replacing duplicates with an empty string
+        result = test_gen.process_constraints().replace({"":pd.NA}).dropna()
         
         self.assertFalse(result.query("B=='spam' & C == True")["A"].duplicated().any())
         self.assertFalse(result.query("B=='spam' & C == False")["A"].duplicated().any())
@@ -727,7 +734,9 @@ class constraintsTests(unittest.TestCase):
             "columns" : {
                 "A" : {
                     "uniques" : 5,
-                    "original_values" : pd.DataFrame(data={"A": list("ABCDE")})
+                    "original_values" : pd.DataFrame(data={
+                        "A": list("ABCDE") + [MISSING_DATA_STR]
+                        })
                 },
                 "C" : {
                     "uniques" : 20,
@@ -759,7 +768,8 @@ class constraintsTests(unittest.TestCase):
         })
 
         test_gen = tm.ConstraintHandler(test_dict, test_data)
-        result = test_gen.process_constraints()
+        # remember for make_distinct we're replacing duplicates with an empty string
+        result = test_gen.process_constraints().replace({"":pd.NA}).dropna()
         
         self.assertFalse(result.query("B=='spam'")["A"].dropna().duplicated().any())
 
@@ -809,7 +819,8 @@ class constraintsTests(unittest.TestCase):
             })
 
             test_gen = tm.ConstraintHandler(test_dict, test_data)
-            result = test_gen.process_constraints()
+            # remember for make_distinct we're replacing duplicates with an empty string
+            result = test_gen.process_constraints().replace({"":pd.NA}).dropna()
 
             self.assertFalse(result.query("B=='spam'")["A"].dropna().duplicated().any())
 
@@ -899,6 +910,129 @@ class constraintsTests(unittest.TestCase):
         self.assertTrue((result.query("B=='spam' & C == True")["A"] == "A").all())
         self.assertTrue((result.query("B=='spam' & C == False")["A"] == "B").all())
 
+    def test_custom_constraints_generate_as_sequence(self):
+        '''
+        Doc string
+        '''
+
+        test_dict = {
+
+            "_rng" : np.random.default_rng(seed=3),
+            "columns" : {
+                "A" : {
+                    "uniques" : 5,
+                    "original_values" : pd.DataFrame(data={
+                        "A": list("ABCD"),
+                        "probability_vector": [0.25, 0.25, 0.25, 0.25]
+                        })
+                },
+            },
+            "constraints" : {
+                "custom_constraints": {
+                    "cc1" : {
+                        "filter"    : "B == 'spam'",
+                        "targets" : {
+                            "A" : "generate_as_sequence",
+                        }
+                    }
+                }
+            },
+        }
+
+        test_data = pd.DataFrame(data={
+            "A" : list("ABCDA") * 2,
+            "B" : ["spam", "spam", "ham", "ham", "ham"] * 2,
+        })
+
+        test_gen = tm.ConstraintHandler(test_dict, test_data)
+        anon_df = test_gen.process_constraints()
+        
+        self.assertTrue(anon_df.query("B=='spam'")["A"].is_monotonic_increasing)
+
+    def test_custom_constraints_generate_as_sequence_with_partition(self):
+        '''
+        Doc string
+        '''
+
+        test_dict = {
+
+            "_rng" : np.random.default_rng(seed=0),
+            "columns" : {
+                "A" : {
+                    "uniques" : 4,
+                    "original_values" : pd.DataFrame(data={
+                        "A": list("ABCD"),
+                        "probability_vector": [0.1, 0.5, 0.4, 0.3]
+                        })
+                },
+            },
+            "constraints" : {
+                "custom_constraints": {
+                    "cc1" : {
+                        "partition" : "B",
+                        "targets" : {
+                            "A" : "generate_as_sequence",
+                        }
+                    }
+                }
+            },
+        }
+
+        test_data = pd.DataFrame(data={
+            "A" : list("ABCDA") * 2,
+            "B" : ["padded"] * 5 + ["complete"] * 4 + ["incomplete"] * 1
+        })
+
+        test_gen = tm.ConstraintHandler(test_dict, test_data)
+        anon_df = test_gen.process_constraints()
+        
+        self.assertTrue(anon_df.query("B=='padded'")["A"].is_monotonic_increasing)
+        self.assertTrue(anon_df.query("B=='complete'")["A"].is_monotonic_increasing)
+        self.assertTrue(anon_df.query("B=='incomplete'")["A"].is_monotonic_increasing)
+
+
+    def test_custom_constraints_generate_as_sequence_with_partition_equal_probs(self):
+        '''
+        Doc string
+        '''
+
+        test_dict = {
+
+            "_rng" : np.random.default_rng(seed=0),
+            "columns" : {
+                "A" : {
+                    "uniques" : 4,
+                    "original_values" : pd.DataFrame(data={
+                        "A": list("ABCD"),
+                        "probability_vector": [0.25, 0.25, 0.25, 0.25]
+                        })
+                },
+            },
+            "constraints" : {
+                "custom_constraints": {
+                    "cc1" : {
+                        "partition" : "B",
+                        "targets" : {
+                            "A" : "generate_as_sequence",
+                        }
+                    }
+                }
+            },
+        }
+
+        test_data = pd.DataFrame(data={
+            "A" : list("ABCDA") * 2,
+            "B" : ["padded"] * 5 + ["complete"] * 4 + ["incomplete"] * 1
+        })
+
+        test_gen = tm.ConstraintHandler(test_dict, test_data)
+        anon_df = test_gen.process_constraints()
+        
+        expected = ["A","B","C","D","","A","B","C","D","A"]
+        result = anon_df["A"].tolist()
+
+        self.assertListEqual(expected, result)
+
 if __name__ == "__main__" and __package__ is None:
     #overwrite __package__ builtin as per PEP 366
     __package__ = "exhibit"
