@@ -26,6 +26,7 @@ from exhibit.core import exhibit  as tm
 def replace_nested_dict_values(d1, d2):
     '''
     Recursive replacement of dictionary values in matching keys
+    or adding new ones.
     '''
 
     for key2 in d2:
@@ -34,6 +35,8 @@ def replace_nested_dict_values(d1, d2):
                 replace_nested_dict_values(d1[key2], d2[key2])
             else:
                 d1[key2] = d2[key2]
+        else:
+            d1[key2] = d2[key2]
 
 def temp_exhibit(
     filename="inpatients.csv",
@@ -296,6 +299,10 @@ class referenceTests(unittest.TestCase):
             - anonymisation method is "random"
             - non-linked categorical column (Sex) has missing data
             - linked columns share missing categorical data
+
+        Because by default the spec includes the basic constraints of los >= avlos,
+        if avlos is null (0.065 probability in source data) then los will also be null
+        in ~130 records.
         '''
 
         source_data_path = Path(package_dir("sample", "_data", "inpatients.csv"))
@@ -690,7 +697,7 @@ class referenceTests(unittest.TestCase):
         '''
         What this reference test is covering:
             - uuid column type
-            - make_distinct, make_same and sorting custom actions
+            - generate_as_sequence, make_same and sorting custom actions
             - no db
         '''
         
